/*
 * @description: 安全巡检-审批
 * @Author: LanJianRong
 * @Date: 2020-12-18
 * @FilePath: \construction_management\web\viewmodels\quality_audit.go
 */
package viewmodels

import (
	"time"

	validation "github.com/go-ozzo/ozzo-validation/v3"
)

type QualityAudit struct {
	Id            string    `form:"id" json:"id"`
	QualityId     string    `form:"quality_id" json:"quality_id"`
	BidsectionId  string    `form:"bidsection_id" json:"bidsection_id"`
	Times         int       `form:"times" json:"times"`
	AuditId       string    `form:"audit_id" json:"audit_id"`
	AuditOrder    int       `form:"audit_order" json:"audit_order"`
	Status        int       `form:"status" json:"status"`
	CreateTime    time.Time `form:"create_time" json:"create_time"`
	EndTime       time.Time `form:"end_time" json:"end_time"`
	Progress      string    `form:"progress" json:"progress"`
	Opinion       string    `form:"opinion" json:"opinion"`
	RectifiedInfo string    `form:"rectifiedInfo" json:"rectifiedInfo"`
	RectifiedTime string    `from:"rectifiedTime" json:"rectifiedTime"`
}

type HistoryQualityAudit struct {
	Id         string    `from:"id" json:"id"`
	Name       string    `from:"name" json:"name"`
	Position   string    `from:"position" json:"position"`
	CreateTime time.Time `from:"create_time" json:"create_time"`
	Opinion    string    `from:"opinion" json:"opinion"`
	Status     int       `from:"status" json:"status"`
	Progress   string    `from:"progress" json:"progress"`
}

func (l QualityAudit) ValidateQualityAddAuditor() error {
	return validation.ValidateStruct(&l,
		validation.Field(&l.BidsectionId, validation.Required.Error("标段ID不能为空")),
		validation.Field(&l.QualityId, validation.Required.Error("巡检ID不能为空")),
		validation.Field(&l.Times, validation.Required.Error("times不能为空")),
		validation.Field(&l.AuditId, validation.Required.Error("审核人不能为空")),
	)
}

func (l QualityAudit) ValidateQualityClose() error {
	return validation.ValidateStruct(&l,
		validation.Field(&l.Id, validation.Required.Error("记录id不能为空")),
		validation.Field(&l.QualityId, validation.Required.Error("巡检id不能为空")),
	)
}

func (l QualityAudit) ValidateQualityBack() error {
	return validation.ValidateStruct(&l,
		validation.Field(&l.Id, validation.Required.Error("记录id不能为空")),
		validation.Field(&l.QualityId, validation.Required.Error("巡检id不能为空")),
	)
}
func (l QualityAudit) ValidateQualityPass() error {
	return validation.ValidateStruct(&l,
		validation.Field(&l.Id, validation.Required.Error("记录id不能为空")),
		validation.Field(&l.QualityId, validation.Required.Error("巡检id不能为空")),
	)
}

type QualityAuditStart struct {
	QualityId        string    `form:"quality_id" json:"quality_id"`
	BidsectionId     string    `form:"bidsection_id" json:"bidsection_id"`
	Times            int       `form:"times" json:"times"`
	Auditors         []string  `form:"auditors" json:"auditors"`
	ReAuditors       []string  `form:"reAuditors" json:"reAuditors"`
	Inspection       string    `from:"inspection" json:"inspection"`
	InspectionDetail string    `from:"inspectionDetail" json:"inspectionDetail"`
	Demand           string    `from:"demand" json:"demand"`
	CreateTime       time.Time `from:"createTime" json:"createTime"`
}

func (l QualityAuditStart) ValidateQualityStart() error {
	return validation.ValidateStruct(&l,
		validation.Field(&l.BidsectionId, validation.Required.Error("标段id不能为空")),
		validation.Field(&l.QualityId, validation.Required.Error("巡检id不能为空")),
		validation.Field(&l.Times, validation.Required.Error("审批次数不能为空")),
		validation.Field(&l.Auditors, validation.Required.Error("审批组不能为空")),
		validation.Field(&l.ReAuditors, validation.Required.Error("复查组不能为空")),
		validation.Field(&l.Inspection, validation.Required.Error("检查项目不能为空")),
		validation.Field(&l.InspectionDetail, validation.Required.Error("现场检查不能为空")),
		validation.Field(&l.Demand, validation.Required.Error("处理措施不能为空")),
		validation.Field(&l.CreateTime, validation.Required.Error("检查日期不能为空")),
	)
}
